unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, SysTray, ExtCtrls, Menus, StdCtrls, WerControl, ComCtrls, 
  DateUtils, RegConfig;

type
  TPingStatus = (psOK,
    psRequestTimedOut,           // - 999
    psConnectionError,           // -1999
    psDestinationUnreachable,    // -2999
    psTTLLimitExceeded,          // -3999
    psNetworkFailure,            // -4999
    psInternalError);            // -5999

  TPingResult = record
    Status: TPingStatus;
    ErrorString: string;
    Time: Integer;
    end;

  TMainForm = class(TForm)
    TrayIcon: TSysTray;
    PingTimer: TTimer;
    Menu: TPopupMenu;
    MenuExit: TMenuItem;
    Label1: TLabel;
    Host: TEdit;
    Label2: TLabel;
    PingFrequencyUpDown: TUpDown;
    PingFrequency: TWerSpinEdit;
    Label3: TLabel;
    PingTimeoutUpDown: TUpDown;
    PingTimeout: TWerSpinEdit;
    Label4: TLabel;
    IconUpdateUpDown: TUpDown;
    IconUpdate: TWerSpinEdit;
    N1: TMenuItem;
    N2: TMenuItem;
    MenuConfiguration: TMenuItem;
    IconTimer: TTimer;
    procedure FormClose(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormDestroy(Sender: TObject);
    procedure PingTimeoutUpDownChangingEx(Sender: TObject;
      var AllowChange: Boolean; NewValue: Smallint;
      Direction: TUpDownDirection);
    procedure PingFrequencyUpDownChangingEx(Sender: TObject;
      var AllowChange: Boolean; NewValue: Smallint;
      Direction: TUpDownDirection);
    procedure FormCreate(Sender: TObject);
    procedure PingTimerTimer(Sender: TObject);
    procedure IconTimerTimer(Sender: TObject);
    procedure MenuExitClick(Sender: TObject);
    procedure MenuAboutClick(Sender: TObject);
    procedure TrayIconDblClick(Sender: TObject);
  private
    { Private declarations }
  public
  published
    procedure DrawIcon;
    procedure PingResult(Status: TPingStatus; ErrorString: string=''; Time: Integer=0);
    { Public declarations }
  end;

var
  MainForm: TMainForm;
  PingStart: TDateTime;

implementation
uses
  PingerU;

{$R *.dfm}

var
  Results: array[0..2000]of TPingResult;
  Mask, Graph: TBitmap;
  IconInfo: TIconInfo;

const
  ErrorColors: array[TPingStatus]of TColor=(
    clBlack, clRed, clFuchsia, clBlue, clAqua, clGray, clYellow);
  ErrorMsgs: array[TPingStatus]of string=(
    '',
    'Request timed out',
    'Connection error',
    'Destination unreachable',
    'TTL limit exceeded',
    'Network failure',
    'Internal error');

// -------------------- graphics ----------------------

function Ipol(r:real;b1,b2:Integer):Integer;
begin
  Result:=b1+Round((integer(b2)-b1)*r);
  if b1<b2 then
    begin
    if Result<b1 then Result:=b1;
    if Result>b2 then Result:=b2;
    end
  else
    begin
    if Result>b1 then Result:=b1;
    if Result<b2 then Result:=b2;
    end
end;

function ColIpol(r:real;c1,c2:TColor):TColor;
begin
  Result:=RGB(Ipol(r,GetRValue(c1),GetRValue(c2)),
              Ipol(r,GetGValue(c1),GetGValue(c2)),
              Ipol(r,GetBValue(c1),GetBValue(c2)));
end;

function GetPingColor(R: Real): TColor;
begin
  if R<0.5 then
    Result:=ColIPol(R*2, clLime, clYellow)
  else
    Result:=ColIPol((R-0.5)*1.8, clYellow, clRed);
end;

procedure TMainForm.DrawIcon;
var
  OldIcon: HICON;
  I, J: Integer;
  C: TColor;
  PixelTime: Integer;
begin
  PixelTime := PingTimeout.IntValue div 16;
  with Graph.Canvas do
    for I:=0 to 15 do
      begin
      C:=GetPingColor(Results[15-I].Time / PingTimeout.IntValue);
      for J:=0 to 15 do
        if Results[15-I].Status<>psOK then
          Pixels[i,j]:=ErrorColors[Results[15-I].Status]
        else
        if Results[15-I].Time > PingTimeout.IntValue then
          Pixels[i,j]:=ErrorColors[psRequestTimedOut]
        else
        if (((15-J)+1)*PixelTime) < Results[15-I].Time then
          Pixels[i,j]:=C
        else
        if (((15-J)  )*PixelTime) < Results[15-I].Time then
          Pixels[i,j]:=ColIPol((Results[15-I].Time - (((15-J)  )*PixelTime)) / PixelTime, clBlack, C)
        else
          Pixels[i,j]:=clBlack;
      end;

  OldIcon:=TrayIcon.Icon.Handle;
  TrayIcon.Icon.Handle:=CreateIconIndirect(IconInfo);
  DestroyIcon(OldIcon);
  if Results[0].Status<>psOK then
    if Results[0].ErrorString='' then
      TrayIcon.Hint:=ErrorMsgs[Results[0].Status]
    else
      TrayIcon.Hint:=ErrorMsgs[Results[0].Status]+' ('+Results[0].ErrorString+')'
  else
  if Results[0].Time=0 then
    TrayIcon.Hint:='WER InetMonitor 2'
  else
    TrayIcon.Hint:='WER InetMonitor 2'#13#10'Ping time: '+IntToStr(Results[0].Time)+' ms';
end;

// -------------------- initialization ----------------------

procedure TMainForm.FormCreate(Sender: TObject);
begin
  LoadCtrl(MainForm, false);
  Mask:=TBitmap.Create;
  Mask.Monochrome:=True;
  Mask.Width:=16;
  Mask.Height:=16;
  Mask.Canvas.Brush.Color:=clBlack;
  Mask.Canvas.FillRect(Rect(0,0,16,16));
  Graph:=TBitmap.Create;
  Graph.PixelFormat:=pf32bit;
  Graph.Width:=16;
  Graph.Height:=16;

  IconInfo.fIcon:=True;
  IconInfo.hbmMask:=Mask.Handle;
  IconInfo.hbmColor:=Graph.Handle;
  TrayIcon.Icon.Handle:=CreateIconIndirect(IconInfo);
  DrawIcon;
 
  LoadCtrl(MainForm, false);
  PingTimeoutUpDown.Max := PingFrequency.IntValue;

  PingTimer.Interval := PingFrequency.IntValue;
  PingTimer.Enabled:=True;
  IconTimer.Interval := IconUpdate.IntValue;
  IconTimer.Enabled:=True;
end;

// -------------------- core ----------------------

var
  Pinger: TPinger = nil;

procedure TMainForm.PingTimerTimer(Sender: TObject);
var I:Integer;
begin
  if Pinger<>nil then
    begin
    Pinger.Expired:=True;
    Pinger:=nil;
    Results[0].Status:=psNetworkFailure;
    Results[0].ErrorString:='thread time-out'; 
    Results[0].Time:=0;
    end;

  for I:=High(Results) downto 1 do
    Results[I]:=Results[I-1];
  Results[0].Status:=psOK;
  Results[0].ErrorString:='';
  Results[0].Time:=0;
  PingStart := Now;

  Pinger:=TPinger.Create(False);
  DrawIcon;
end;

var
  PingFinished: Boolean = false;

procedure TMainForm.IconTimerTimer(Sender: TObject);
begin
  PingTimer.Interval:=PingFrequency.IntValue;
  IconTimer.Interval:=IconUpdate.IntValue;

  if(Pinger<>nil)or PingFinished then
    begin

    if Pinger<>nil then
      Results[0].Time := MilliSecondsBetween(Now, PingStart);

    try
      DrawIcon;
    except
      on E:Exception do
        MessageBox(0,PChar(E.Message),nil,0);
      end;
    end;

  PingFinished:=False;
end;

procedure TMainForm.PingResult(Status: TPingStatus; ErrorString: string=''; Time: Integer=0);
begin
  Pinger := nil;
  Results[0].Status:=Status;
  Results[0].ErrorString:=ErrorString;
  Results[0].Time:=Time;
  //DrawIcon;
  PingFinished:=True;
end;

// ------------------- interface ----------------------

procedure TMainForm.MenuExitClick(Sender: TObject);
begin
  Application.Terminate;
end;

procedure TMainForm.MenuAboutClick(Sender: TObject);
begin
  ShowMessage('InetMonitor 2.1'#13#10'by Vladimir Panteleev'#13#10'<thecybershadow@gmail.com>');
end;

procedure TMainForm.PingTimeoutUpDownChangingEx(Sender: TObject;
  var AllowChange: Boolean; NewValue: Smallint; Direction: TUpDownDirection);
begin
  {PingFrequencyUpDown.Max := NewValue;
  if PingFrequency.IntValue>NewValue then
    PingFrequency.IntValue:=NewValue;
  AllowChange := True;}
  ShowMessage('t');
end;

procedure TMainForm.PingFrequencyUpDownChangingEx(Sender: TObject;
  var AllowChange: Boolean; NewValue: Smallint; Direction: TUpDownDirection);
begin
  PingTimeoutUpDown.Max := NewValue;
  if PingTimeout.IntValue>NewValue then
    PingTimeout.IntValue:=NewValue;
  AllowChange := True;
  ShowMessage('t');
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  SaveCtrl(MainForm, false);
end;

procedure TMainForm.FormClose(Sender: TObject);
begin
  //if Visible then
  //  begin
  //  Hide;
  //  Abort;
  //  end;
end;

procedure TMainForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  //Hide;
  //CanClose:=False;
end;

procedure TMainForm.TrayIconDblClick(Sender: TObject);
begin
  Show;
  Application.Restore;
  SetForegroundWindow(Handle);
end;

end.
