unit PingerU;

interface

uses
  Classes;

type
  TPinger = class(TThread)
  public
    Expired: boolean;
  protected
    procedure Execute; override;
  end;

implementation
uses
  Main, Windows, SysUtils, WinSock, DateUtils;

type
  TSunB = packed record
    s_b1, s_b2, s_b3, s_b4: byte;
  end;

  TSunW = packed record
    s_w1, s_w2: word;
  end;

  PIPAddr = ^TIPAddr;
  TIPAddr = record
    case integer of
      0: (S_un_b: TSunB);
      1: (S_un_w: TSunW);
      2: (S_addr: longword);
  end;
  IPAddr = TIPAddr;

  IP_OPTION_INFORMATION = record
    Ttl : byte;
    Tos : byte;
    Flags : byte;
    OptionsSize : byte;
    OptionsData : pointer;
    end;

  ICMP_ECHO_REPLY = record
    Address : IPAddr;
    Status : ULONG;
    RoundTripTime : ULONG;
    DataSize : Word;
    Reserved : Word;
    Data : Pointer;
    Options : IP_OPTION_INFORMATION;
    end;
 
function IcmpCreateFile : THandle; stdcall; external 'icmp.dll';
function IcmpCloseHandle (icmpHandle : THandle) : boolean; stdcall; external 'icmp.dll';
function IcmpSendEcho (
    IcmpHandle : THandle; 
    DestinationAddress : IPAddr;
    RequestData : Pointer; 
    RequestSize : Smallint;
    RequestOptions : pointer;
    ReplyBuffer : Pointer;
    ReplySize : DWORD;
    Timeout : DWORD
    ) : DWORD; stdcall; external 'icmp.dll';

procedure TranslateStringToTInAddr(AIP: string; var AInAddr);
var
  phe: PHostEnt;
  pac: PChar;
  GInitData: TWSAData;
begin
  WSAStartup($101, GInitData);
  try
    phe := GetHostByName(PChar(AIP));
    if Assigned(phe) then
    begin
      pac := phe^.h_addr_list^;
      if Assigned(pac) then
      begin
        with TIPAddr(AInAddr).S_un_b do begin
          s_b1 := Byte(pac[0]);
          s_b2 := Byte(pac[1]);
          s_b3 := Byte(pac[2]);
          s_b4 := Byte(pac[3]);
        end;
      end
      else
      begin
        raise Exception.Create('Error getting IP from HostName');
      end;
    end
    else
    begin
      raise Exception.Create('Error getting HostName');
    end;
  except
    FillChar(AInAddr, SizeOf(AInAddr), #0);
  end;
  WSACleanup;
end;

function Ping(InetAddress : string; TimeOut: Integer) : Integer;
var
  Handle : THandle;
  InAddr : IPAddr;
  DW : DWORD;
  PingBuf: array[0..31] of char;
  Reply  : ICMP_ECHO_REPLY;
begin
  Handle := IcmpCreateFile;
  Result := 1;
  if Handle = INVALID_HANDLE_VALUE then
    Exit;
  TranslateStringToTInAddr(InetAddress, InAddr);
  DW := IcmpSendEcho(Handle, InAddr, @PingBuf, 32, nil, @reply, SizeOf(icmp_echo_reply) + 32 , TimeOut);
  if DW=0 then
    Result := GetLastError
  else
    Result := 0;
  IcmpCloseHandle(Handle);
end;

{ TPinger }

{
11001 Buffer Too Small 
11002 Destination Net Unreachable 
11003 Destination Host Unreachable 
11004 Destination Protocol Unreachable 
11005 Destination Port Unreachable 
11006 No Resources 
11007 Bad Option 
11008 Hardware Error 
11009 Packet Too Big 
11010 Request Timed Out 
11011 Bad Request 
11012 Bad Route 
11013 TimeToLive Expired Transit 
11014 TimeToLive Expired Reassembly 
11015 Parameter Problem 
11016 Source Quench 
11017 Option Too Big 
11018 Bad Destination 
11032 Negotiating IPSEC 
11050 General Failure 
}

procedure TPinger.Execute;
var
  Error : Integer;
begin
  Expired:=False;
  FreeOnTerminate:=True;
  with MainForm do
    try
      Error := Ping(MainForm.Host.Text, MainForm.PingTimeout.IntValue-50);
      
      if Expired then 
        Exit;

      if Error=0 then
        PingResult(psOK, '', MilliSecondsBetween(Now, PingStart))
      else
       case Error of
        11010:
          PingResult(psRequestTimedOut);
        11008, 11011, 11012:
          PingResult(psConnectionError);
        11002..11005:
          PingResult(psDestinationUnreachable);
        11013, 11014:
          PingResult(psTTLLimitExceeded);
        else
          PingResult(psInternalError, 'error code '+IntToStr(Error));
        end;
    except
      on E: Exception do
        MainForm.PingResult(psInternalError, E.Message);
      end;
end;

end.
